@@ -19,11 +19,12 @@ import functools
 import inspect
 import socket
 import threading
-import uuid
 
 from oslo_utils import reflection
+from oslo_utils import uuidutils
 import six
 
+from osprofiler import _utils as utils
 from osprofiler import notifier
 
 
@@ -31,7 +32,7 @@ from osprofiler import notifier
 __local_ctx = threading.local()
 
 
-def _clean():
+def clean():
     __local_ctx.profiler = None
 
 
@@ -45,8 +46,7 @@ def _ensure_no_multiple_traced(traceable_attrs):
                              % (attr_name, traced_times))
 
 
-def init(hmac_key, base_id=None, parent_id=None, connection_str=None,
-         project=None, service=None):
+def init(hmac_key, base_id=None, parent_id=None):
     """Init profiler instance for current thread.
 
     You should call profiler.init() before using osprofiler.
@@ -55,16 +55,11 @@ def init(hmac_key, base_id=None, parent_id=None, connection_str=None,
     :param hmac_key: secret key to sign trace information.
     :param base_id: Used to bind all related traces.
     :param parent_id: Used to build tree of traces.
-    :param connection_str: Connection string to the backend to use for
-                           notifications.
-    :param project: Project name that is under profiling
-    :param service: Service name that is under profiling
     :returns: Profiler instance
     """
-    __local_ctx.profiler = _Profiler(hmac_key, base_id=base_id,
-                                     parent_id=parent_id,
-                                     connection_str=connection_str,
-                                     project=project, service=service)
+    if get() is None:
+        __local_ctx.profiler = _Profiler(hmac_key, base_id=base_id,
+                                         parent_id=parent_id)
     return __local_ctx.profiler
 
 
@@ -95,7 +90,8 @@ def stop(info=None):
         profiler.stop(info=info)
 
 
-def trace(name, info=None, hide_args=False, allow_multiple_trace=True):
+def trace(name, info=None, hide_args=False, hide_result=True,
+          allow_multiple_trace=True):
     """Trace decorator for functions.
 
     Very useful if you would like to add trace point on existing function:
@@ -110,6 +106,9 @@ def trace(name, info=None, hide_args=False, allow_multiple_trace=True):
     :param hide_args: Don't push to trace info args and kwargs. Quite useful
                       if you have some info in args that you wont to share,
                       e.g. passwords.
+    :param hide_result: Boolean value to hide/show function result in trace.
+                        True - hide function result (default).
+                        False - show function result in trace.
     :param allow_multiple_trace: If the wrapped function has already been
                                  traced either allow the new trace to occur
                                  or raise a value error denoting that multiple
@@ -141,24 +140,46 @@ def trace(name, info=None, hide_args=False, allow_multiple_trace=True):
 
         @functools.wraps(f)
         def wrapper(*args, **kwargs):
-            if "name" not in info["function"]:
+            # NOTE(tovin07): Workaround for this issue
+            # F823 local variable 'info'
+            # (defined in enclosing scope on line xxx)
+            # referenced before assignment
+            info_ = info
+            if "name" not in info_["function"]:
                 # Get this once (as it should **not** be changing in
                 # subsequent calls).
-                info["function"]["name"] = reflection.get_callable_name(f)
+                info_["function"]["name"] = reflection.get_callable_name(f)
 
             if not hide_args:
-                info["function"]["args"] = str(args)
-                info["function"]["kwargs"] = str(kwargs)
+                info_["function"]["args"] = str(args)
+                info_["function"]["kwargs"] = str(kwargs)
 
-            with Trace(name, info=info):
-                return f(*args, **kwargs)
+            stop_info = None
+            try:
+                start(name, info=info_)
+                result = f(*args, **kwargs)
+            except Exception as ex:
+                stop_info = {
+                    "etype": reflection.get_class_name(ex),
+                    "message": six.text_type(ex)
+                }
+                raise
+            else:
+                if not hide_result:
+                    stop_info = {"function": {"result": repr(result)}}
+                return result
+            finally:
+                if stop_info:
+                    stop(info=stop_info)
+                else:
+                    stop()
 
         return wrapper
 
     return decorator
 
 
-def trace_cls(name, info=None, hide_args=False,
+def trace_cls(name, info=None, hide_args=False, hide_result=True,
               trace_private=False, allow_multiple_trace=True,
               trace_class_methods=False, trace_static_methods=False):
     """Trace decorator for instances of class .
@@ -181,6 +202,9 @@ def trace_cls(name, info=None, hide_args=False,
     :param hide_args: Don't push to trace info args and kwargs. Quite useful
                       if you have some info in args that you wont to share,
                       e.g. passwords.
+    :param hide_result: Boolean value to hide/show function result in trace.
+                        True - hide function result (default).
+                        False - show function result in trace.
     :param trace_private: Trace methods that starts with "_". It wont trace
                           methods that starts "__" even if it is turned on.
     :param trace_static_methods: Trace staticmethods. This may be prone to
@@ -234,7 +258,8 @@ def trace_cls(name, info=None, hide_args=False,
             # halfway trace this class).
             _ensure_no_multiple_traced(traceable_attrs)
         for i, (attr_name, attr) in enumerate(traceable_attrs):
-            wrapped_method = trace(name, info=info, hide_args=hide_args)(attr)
+            wrapped_method = trace(name, info=info, hide_args=hide_args,
+                                   hide_result=hide_result)(attr)
             wrapper = traceable_wrappers[i]
             if wrapper is not None:
                 wrapped_method = wrapper(wrapped_method)
@@ -254,6 +279,7 @@ class TracedMeta(type):
     >>>      __trace_args__ = {'name': 'rpc',
     >>>                        'info': None,
     >>>                        'hide_args': False,
+    >>>                        'hide_result': True,
     >>>                        'trace_private': False}
     >>>
     >>>      def my_method(self, some_args):
@@ -279,9 +305,9 @@ class TracedMeta(type):
                             "e.g. __trace_args__ = {'name': 'rpc'}")
 
         traceable_attrs = []
-        for attr_name, attr_value in six.iteritems(attrs):
-            if not (inspect.ismethod(attr_value) or
-                    inspect.isfunction(attr_value)):
+        for attr_name, attr_value in attrs.items():
+            if not (inspect.ismethod(attr_value)
+                    or inspect.isfunction(attr_value)):
                 continue
             if attr_name.startswith("__"):
                 continue
@@ -322,7 +348,10 @@ class Trace(object):
 
     def __exit__(self, etype, value, traceback):
         if etype:
-            info = {"etype": reflection.get_class_name(etype)}
+            info = {
+                "etype": reflection.get_class_name(etype),
+                "message": value.args[0] if value.args else None
+            }
             stop(info=info)
         else:
             stop()
@@ -330,17 +359,21 @@ class Trace(object):
 
 class _Profiler(object):
 
-    def __init__(self, hmac_key, base_id=None, parent_id=None,
-                 connection_str=None, project=None, service=None):
+    def __init__(self, hmac_key, base_id=None, parent_id=None):
         self.hmac_key = hmac_key
         if not base_id:
-            base_id = str(uuid.uuid4())
+            base_id = str(uuidutils.generate_uuid())
         self._trace_stack = collections.deque([base_id, parent_id or base_id])
         self._name = collections.deque()
         self._host = socket.gethostname()
-        self._connection_str = connection_str
-        self._project = project
-        self._service = service
+
+    def get_shorten_id(self, uuid_id):
+        """Return shorten id of a uuid that will be used in OpenTracing drivers
+
+        :param uuid_id: A string of uuid that was generated by uuidutils
+        :returns: A shorter 64-bit long id
+        """
+        return format(utils.shorten_id(uuid_id), "x")
 
     def get_base_id(self):
         """Return base id of a trace.
@@ -362,7 +395,7 @@ class _Profiler(object):
         """Start new event.
 
         Adds new trace_id to trace stack and sends notification
-        to collector (may be ceilometer). With "info" and 3 ids:
+        to collector. With "info" and 3 ids:
         base_id - to be able to retrieve all trace elements by one query
         parent_id - to build tree of events (not just a list)
         trace_id - current event id.
@@ -374,10 +407,8 @@ class _Profiler(object):
 
         info = info or {}
         info["host"] = self._host
-        info["project"] = self._project
-        info["service"] = self._service
         self._name.append(name)
-        self._trace_stack.append(str(uuid.uuid4()))
+        self._trace_stack.append(str(uuidutils.generate_uuid()))
         self._notify("%s-start" % name, info)
 
     def stop(self, info=None):
@@ -389,8 +420,6 @@ class _Profiler(object):
         """
         info = info or {}
         info["host"] = self._host
-        info["project"] = self._project
-        info["service"] = self._service
         self._notify("%s-stop" % self._name.pop(), info)
         self._trace_stack.pop()
 
